package com.citizen.app.devicecontrolsample1;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.os.Build;
import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;
import android.widget.Spinner;
import android.widget.AdapterView;

import com.citizen.sdk.CitizenPrinterInfo;
import com.citizen.sdk.DataListener;
import com.citizen.sdk.ESCPOSConst;
import com.citizen.sdk.ESCPOSPrinter;
import com.citizen.sdk.LineDisplay;
import com.citizen.sdk.LineDisplayConst;
import com.citizen.sdk.Scanner;
import com.citizen.sdk.ScannerConst;
import com.citizen.sdk.StatusUpdateListener;

import java.text.SimpleDateFormat;
import java.util.Date;

import	java.util.List;
import	java.util.ArrayList;

public class MainActivity extends Activity {
    private ESCPOSPrinter printer;
    private LineDisplay display;
    private Scanner scanner;
    private int scanCnt = 0;
    String msg;

    final String PREFERENCES_NAME = "Device Control Sample1";
    final String PRE_KEY_IP_ADDRESS = "IP_Address";
    final String DEFAULT_IP_ADDRESS = "192.168.129.222";
    final String PRE_KEY_BLUETOOTH_ADDRESS = "Bluetooth_Address";
    final String DEFAULT_BLUETOOTH_ADDRESS = "88:6B:0F:05:57:DF";
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);        //Hide soft keyboard

        getPermissions(permissions());

        printer = new ESCPOSPrinter();
        display = new LineDisplay();
        scanner = new Scanner();


        EditText editText_Port = findViewById(R.id.editText_Port);
        editText_Port.setText("9100");

        EditText editText_DisplayPort = findViewById(R.id.editText_DisplayPort);
        editText_DisplayPort.setText("9200");

        EditText editText_ScannerPort = findViewById(R.id.editText_ScannerPort);
        editText_ScannerPort.setText("9210");


       // Select IF
        final Spinner spinner = findViewById(R.id.spinner);
        spinner.setSelection(1);             //Default(Bluetooth)

        if (spinner != null) {
            spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
                @Override
                public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                    //String address = "";

                    int connectType = getSelectedInterface();
                    //if (ESCPOSConst.CMP_PORT_USB == connectType) {
                    //    address = "null(Automatic Detection)";
                    //}
                    setAddress(loadAddress( connectType ));
                }

                @Override
                public void onNothingSelected(AdapterView<?> parent) {
                    Log.e("OnNothingSelected", "NothingSelected");
                }
            });
        }

        // Search
        final Button button_Search = findViewById(R.id.button_Search);
        if (button_Search != null) {
            button_Search.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    int searchMode = 0;     // SearchMode == 0  :   SearchCitizenPrinter
                    //            != 0  :   SearchLabelPrinter
                    int connectType;
                    int timeout;
                    int[] errCode = new int[1];

                    // Get selected interface
                    connectType = getSelectedInterface();
                    if (connectType == ESCPOSConst.CMP_PORT_USB) {
                        // Usb interface is Auto Detect only
                        Toast.makeText(MainActivity.this, "USB interface is Automatic Detection only", Toast.LENGTH_LONG).show();
                        return;
                    }
                    // Set context
                    printer.setContext(MainActivity.this);

                    // Get timeout
                    timeout = getTimeOut(connectType);
                    // Search
                    if (searchMode == 0) {
                        // Search Citizen Printer
                        CitizenPrinterInfo[] list1 = printer.searchCitizenPrinter(connectType, timeout, errCode);
                        if (list1.length > 0) {
                            final CharSequence[] item = new CharSequence[list1.length];
                            for (int i = 0; i < list1.length; i++) {
                                if (connectType == ESCPOSConst.CMP_PORT_WiFi) {
                                    item[i] = list1[i].ipAddress;
                                } else if (connectType == ESCPOSConst.CMP_PORT_Bluetooth) {
                                    item[i] = list1[i].deviceName;
                                    item[i] = list1[i].bdAddress;
                                }else
                                    item[i] = list1[i].bdAddress;
                            }

                            // Search address preview and select
                            new AlertDialog.Builder(MainActivity.this)
                                    .setTitle("Select the address")
                                    .setItems(item,
                                            new DialogInterface.OnClickListener() {
                                                @Override
                                                public void onClick(DialogInterface dialog, int which) {
                                                    setAddress(String.format("%s", item[which]));
                                                }
                                            })
                                    .show();
                        } else {
                            // 0 search results
                            Toast.makeText(MainActivity.this, "0 Search Results", Toast.LENGTH_LONG).show();
                        }
                    } else {
                        // Search Printer
                        String[] list2 = printer.searchESCPOSPrinter(connectType, timeout, errCode);
                        if (list2.length > 0) {
                            final CharSequence[] item = list2;

                            // Search address preview and select
                            new AlertDialog.Builder(MainActivity.this)
                                    .setTitle("Select the address")
                                    .setItems(item,
                                            new DialogInterface.OnClickListener() {
                                                @Override
                                                public void onClick(DialogInterface dialog, int which) {
                                                    setAddress(String.format("%s", item[which]));
                                                }
                                            })
                                    .show();
                        } else {
                            // 0 search results
                            Toast.makeText(MainActivity.this, "0 Search Results", Toast.LENGTH_LONG).show();
                        }
                    }
                }
            });
        }

        // Button Sample1
        final Button button_Sample1 = findViewById(R.id.button_Sample1);
        button_Sample1.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                int connectType = getSelectedInterface();
                String addr = getAddress();
                String port = getPort();
                int result1;
                int len = port.length();

                if(len == 0){
                    result1 = printer.connect(connectType, addr, 9100);
                }else {
                    result1 = printer.connect(connectType, addr, Integer.parseInt(port));
                }

                if(result1 != ESCPOSConst.CMP_SUCCESS){
                    msg = "Printer connect() Error : " + result1;
                    alertDialog( "Error", msg );
                    return;
                }
                printTest1();
                saveAddress( connectType, addr );
            }
        });

        // Button Sample2
        final Button button_Sample2 = findViewById( R.id.button_Sample2 );
        button_Sample2.setOnClickListener( new View.OnClickListener()
        {
            public void onClick( View v )
            {

                int connectType = getSelectedInterface();
                String addr = getAddress();
                String port = getPort();
                int result1;
                int len = port.length();

                if(len == 0){
                    result1 = printer.connect(connectType, addr, 9100);
                }else {
                    result1 = printer.connect(connectType, addr, Integer.parseInt(port));
                }

                if(result1 != ESCPOSConst.CMP_SUCCESS){
                    msg = "Printer connect() Error : " + result1;
                    alertDialog( "Error", msg );
                    return;
                }
                printTest2();
                saveAddress( connectType, addr );
            }
        } );

        //
        // Button Sample Barcode Print
        final Button button_SampleBarcode = findViewById( R.id.button_SampleBarcode );
        button_SampleBarcode.setOnClickListener( new View.OnClickListener()
        {
            public void onClick( View v )
            {


                int connectType = getSelectedInterface();
                String addr = getAddress();
                String port = getPort();
                int result1;
                int len = port.length();

                if(len == 0){
                    result1 = printer.connect(connectType, addr, 9100);
                }else {
                    result1 = printer.connect(connectType, addr, Integer.parseInt(port));
                }

                if(result1 != ESCPOSConst.CMP_SUCCESS){
                    msg = "Printer connect() Error : " + result1;
                    alertDialog( "Error", msg );
                    return;
                }
                printBarcode();
                printer.disconnect();
                saveAddress( connectType, addr );
            }
        } );


        // Button Display1
        final Button button_SampleDisplay1 = findViewById(R.id.button_SampleDisplay1);
        button_SampleDisplay1.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {


                int connectType = getSelectedInterface();
                String addr = getDisplayAddress();
                String displayport = getDisplayPort();
                int result1;
                int len = displayport.length();

                if(len == 0){
                    result1 = display.connect(connectType, addr, 9200);
                }else {
                    result1 = display.connect(connectType, addr, Integer.parseInt(displayport));
                }

                if (LineDisplayConst.CDP_SUCCESS != result1) {
                    if (result1 == LineDisplayConst.CDP_E_CONNECTED)
                        msg = "Error : Already connected";
                    else
                        msg = "LineDisplay connect() Error : " + result1;

                    alertDialog("Error", msg);
                    return;
                }

                displayTest1();

                saveAddress( connectType, addr );
            }
        });

        // Button Display2
        final Button button_SampleDisplay2 = findViewById(R.id.button_SampleDisplay2);
        button_SampleDisplay2.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {


                int connectType = getSelectedInterface();
                String addr = getDisplayAddress();
                String displayport = getDisplayPort();
                int result1;
                int len = displayport.length();

                if(len == 0){
                    result1 = display.connect(connectType, addr, 9200);
                }else {
                    result1 = display.connect(connectType, addr, Integer.parseInt(displayport));
                }

                if (LineDisplayConst.CDP_SUCCESS != result1) {
                    if (result1 == LineDisplayConst.CDP_E_CONNECTED)
                        msg = "Error : Already connected";
                    else
                        msg = "LineDisplay connect() Error : " + result1;

                    alertDialog("Error", msg);
                    return;
                }
                displayTest2();
                saveAddress( connectType, addr );
            }
        });

        //
        // Button ScannerConnect
        final Button button_ScannerConnect = findViewById(R.id.button_ScannerConnect);
        button_ScannerConnect.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                int connectType = getSelectedInterface();
                String addr = getScannerAddress();
                String scannerport = getScannerPort();

                int result1;
                int len = scannerport.length();

                if(len == 0){
                    result1 = scanner.connect(connectType, addr, 9100);
                }else {
                    result1 = scanner.connect(connectType, addr, Integer.parseInt(scannerport));
                }

                if (ScannerConst.CSC_SUCCESS != result1) {
                    if (result1 == ScannerConst.CSC_E_CONNECTED)
                        msg = "Error : Already connected";
                    else
                        msg = "Scanner connect() Error : " + result1;

                    alertDialog("Error", msg);
                    return;
                }

                toast( "Scanner connect() : Success", Toast.LENGTH_SHORT );
                saveAddress( connectType, addr );

            }
        });

        //
        // Button ScannerDisconnect
        final Button button_ScannerDisconnect = findViewById( R.id.button_ScannerDisconnect );
        button_ScannerDisconnect.setOnClickListener( new View.OnClickListener()
        {
            public void onClick( View v )
            {

                String msg;
                EditText editText_Status = findViewById(R.id.editText_Status);

                int result1 = scanner.disconnect();

                if ( result1 == ScannerConst.CSC_SUCCESS)
                {
                    toast( "Scanner disconnect() : Success", Toast.LENGTH_SHORT );
                    editText_Status.setText("STOPPED");
                    editText_Status.setBackgroundColor(Color.WHITE);
                }
                else
                {
                    if (result1 == ScannerConst.CSC_E_DISCONNECT)
                        msg = "Error : Not connected";
                    else
                        msg = "Scanner disconnect() Error : " + result1;

                    alertDialog( "Error", msg );
                }
            }
        } );



        //
        // Call back processing
        DataListener listener = new DataListener(){
            @Override
            public void onDataEvent(final byte[] scanData) {
                Log.d("Scanner", "Call back: " + new String(scanData));
                scanCnt++;

                runOnUiThread(new Runnable() {
                    @Override
                    public synchronized void run() {
                        toast( "Scan Call back: " + new String(scanData), Toast.LENGTH_SHORT );
                    }
                });
            }
        };
        scanner.setDataEventCallback(listener);

        //
        // Call back Status processing
        StatusUpdateListener statuslistener = new StatusUpdateListener(){
            @Override
            public void onStatusUpdateEvent(final int status) {
                runOnUiThread(new Runnable() {
                    @Override
                    public synchronized void run() {
                        Log.d("Scanner", "Status update call back: " + status);
                        EditText editText_Status = findViewById(R.id.editText_Status);

                        if(status == 2001){
                            editText_Status.setText("ONLINE");
                            editText_Status.setBackgroundColor(Color.GREEN);
                        }
                        else if(status == 2002){
                            editText_Status.setText("OFFLINE");
                            editText_Status.setBackgroundColor(Color.RED);
                        }
                    }
                });
            }
        };
        scanner.setStatusUpdateEventCallback(statuslistener);
    }

    private boolean printTest1()
    {
        boolean bRet = true;
        String msg;

        /** Date format */
        SimpleDateFormat mTimeFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
        /** Date information */
        String time = mTimeFormat.format(new Date());

        // Display output
        display.setEncoding("Shift_JIS");
        display.clearDisplay();
        display.displayText(time);
        display.setCursorPosition(1, 2);
        display.displayText("Sample Display");

        // Printer output
        printer.setEncoding("Shift_JIS");
        printer.transactionPrint(ESCPOSConst.CMP_TP_TRANSACTION);

        printer.printText( getString( R.string.app_name ) + "\n\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT );
        printer.printText( "- Sample Print 1 -\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT );
        printer.printText( "1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890\n", ESCPOSConst.CMP_ALIGNMENT_RIGHT, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT );

        // Partial Cut with Pre-Feed
        printer.cutPaper( ESCPOSConst.CMP_CUT_PARTIAL_PREFEED );

        int result1 = printer.transactionPrint(ESCPOSConst.CMP_TP_NORMAL);

        printer.disconnect();

        if ( ESCPOSConst.CMP_SUCCESS != result1){
            bRet = false;
            if (result1 == ESCPOSConst.CMP_E_DISCONNECT)
                msg = "Error : Not connected";
            else
                msg = "Print Error : " + result1;

            alertDialog( "Error", msg );
        }else
            toast(" Success ", Toast.LENGTH_SHORT);

        return bRet;
    }

    //
    // Test output
    private boolean printTest2()
    {
        boolean bRet = true;
        String msg;

        /** Date format */
        SimpleDateFormat mTimeFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
        /** Date information */
        String time = mTimeFormat.format(new Date());

        // Display output
        display.setEncoding("Shift_JIS");
        display.clearDisplay();
        display.displayText(time);
        display.setCursorPosition(1, 2);
        display.displayText("SamoleDisplay");

        // Printer output
        printer.setEncoding("Shift_JIS");
        printer.transactionPrint(ESCPOSConst.CMP_TP_TRANSACTION);

        printer.printText( getString( R.string.app_name ) + "\n\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT );
        printer.printText( "- Sample Print 1 -\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT );
        printer.printText( "1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890\n", ESCPOSConst.CMP_ALIGNMENT_RIGHT, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT );
        // Start Page mode
        printer.pageModePrint( ESCPOSConst.CMP_PM_PAGE_MODE );

        // Position set
        printer.setPageModeVerticalPosition( 0 );
        printer.setPageModeHorizontalPosition( 0 );

        // Direction set
        printer.setPageModePrintDirection( ESCPOSConst.CMP_PD_TOP_TO_BOTTOM );

        // Print data output
        printer.setPageModePrintArea( "308,0,76,800" );

        //posPtr_Bluetooth.printNormal( "\u001b|4C- Receipt -\n" );
        printer.printNormal( "\u001b|4C- Receipt -\n" );
        printer.setPageModePrintArea( "184,0,120,800" );
        printer.printText( "   $ 299.99-  \n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_UNDERLINE | ESCPOSConst.CMP_FNT_BOLD, ESCPOSConst.CMP_TXT_4WIDTH | ESCPOSConst.CMP_TXT_4HEIGHT );
        printer.setPageModePrintArea( "88,0,88,560" );
        printer.printText( "CITIZEN SYSTEMS\n", ESCPOSConst.CMP_ALIGNMENT_RIGHT, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_2WIDTH | ESCPOSConst.CMP_TXT_3HEIGHT );
        printer.setPageModePrintArea( "0,0,88,480" );
        printer.printBarCode( "123456789012", ESCPOSConst.CMP_BCS_UPCA, 64, 4, ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_HRI_TEXT_BELOW );
        printer.setPageModePrintArea( "0,600,192,192" );
        printer.printQRCode( "http://www.citizen-systems.co.jp/", 5, ESCPOSConst.CMP_QRCODE_EC_LEVEL_L, ESCPOSConst.CMP_ALIGNMENT_LEFT );

        // End Page mode
        printer.pageModePrint( ESCPOSConst.CMP_PM_NORMAL );

        // Partial Cut with Pre-Feed
        printer.cutPaper( ESCPOSConst.CMP_CUT_PARTIAL_PREFEED );

        int result1 = printer.transactionPrint(ESCPOSConst.CMP_TP_NORMAL);

        printer.disconnect();

        if ( ESCPOSConst.CMP_SUCCESS != result1){
            bRet = false;
            if (result1 == ESCPOSConst.CMP_E_DISCONNECT)
                msg = "Error : Not connected";
            else
                msg = "Print Error : " + result1;

            alertDialog( "Error", msg );
        }else
            toast(" Success ", Toast.LENGTH_SHORT);

        return bRet;
    }

    // displayTest1
    private boolean displayTest1() {
        boolean bRet = true;
        // Display output
        display.setEncoding("Shift_JIS");
        display.clearDisplay();
        display.displayText("New York steak", true );
        display.moveCursor(-1, 1);
        display.displayText("$89.99-");

        int result1 = display.disconnect();
        if (result1 != LineDisplayConst.CDP_SUCCESS){
            alertDialog( "Not Success:", Integer.toString( result1 ) );
        }
        else
            toast(" Success ", Toast.LENGTH_SHORT);
        return bRet;
    }

    // displayTest2
    private boolean displayTest2() {
        boolean bRet = true;
        // Display output
        display.setEncoding("Shift_JIS");
        display.clearDisplay();
        display.displayText("Thank you so much!");
        display.setCursorPosition(13, 2);
        display.displayText("$299.99-",true);

        int result1 = display.disconnect();
        if (result1 != LineDisplayConst.CDP_SUCCESS){
            alertDialog( "Not Success:", Integer.toString( result1 ) );
        }
        else
            toast(" Success ", Toast.LENGTH_SHORT);

        return bRet;
    }

    // Sample barcode print
    private boolean printBarcode()
    {
        boolean bRet = true;
        String msg;

        int height = 100;
        printer.transactionPrint(ESCPOSConst.CMP_TP_TRANSACTION);
        printer.printBarCode( "123456789012", ESCPOSConst.CMP_BCS_UPCA, height, 4, ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_HRI_TEXT_BELOW );
        printer.printText( "\n\n\n\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, 0, 0 );
        printer.printBarCode( "987654321098", ESCPOSConst.CMP_BCS_UPCA, height, 4, ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_HRI_TEXT_BELOW );
        printer.printText( "\n\n\n\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, 0, 0 );
        printer.printBarCode( "555555555555", ESCPOSConst.CMP_BCS_UPCA, height, 4, ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_HRI_TEXT_BELOW );
        printer.printText( "\n\n\n\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, 0, 0 );
        printer.cutPaper( ESCPOSConst.CMP_CUT_PARTIAL_PREFEED );
        int result = printer.transactionPrint(ESCPOSConst.CMP_TP_NORMAL);

        if ( ESCPOSConst.CMP_SUCCESS != result){
            bRet = false;
            if (result == ESCPOSConst.CMP_E_DISCONNECT)
                msg = "Error : Not connected";
            else
                msg = "transactionPrint() Error : " + result;

            alertDialog( "Error", msg );
        }
        return bRet;
    }

    //
    // Timeout
    private int getTimeOut(int type) {
        int timeout;

        if (ESCPOSConst.CMP_PORT_WiFi == type) {
            timeout = 3;
        } else if ((ESCPOSConst.CMP_PORT_Bluetooth == type) || (ESCPOSConst.CMP_PORT_Bluetooth_Insecure == type)) {
            timeout = 10;
        } else {
            timeout = 0;
        }

        return timeout;
    }
    //
    // Select IF
    private int getSelectedInterface() {
        int selectIF = ESCPOSConst.CMP_PORT_WiFi;
        Spinner spinner = findViewById(R.id.spinner);

        if (spinner != null) {
            selectIF = spinner.getSelectedItemPosition();
        }
        return selectIF;
    }

    //
    //Get Address
    private String getAddress() {
        String address = "";
        EditText editText_Address = findViewById(R.id.editText_Address);

        if (editText_Address != null) {
            address = editText_Address.getText().toString();
        }
        return address;
    }

    //Get Port
    private String getPort() {
        String port = "";
        EditText editText_Port = findViewById(R.id.editText_Port);

        if (editText_Port != null) {
            port = editText_Port.getText().toString();
        }
        return port;
    }

    //
    //Get DisplayAddress
    private String getDisplayAddress() {
        String address = "";
        EditText editText_DisplayAddress = findViewById(R.id.editText_DisplayAddress);

        if (editText_DisplayAddress != null) {
            address = editText_DisplayAddress.getText().toString();
        }
        return address;
    }

    //Get DisplayPort
    private String getDisplayPort() {
        String displayport = "";
        EditText editText_DisplayPort = findViewById(R.id.editText_DisplayPort);

        if (editText_DisplayPort != null) {
            displayport = editText_DisplayPort.getText().toString();
        }
        return displayport;
    }

    //
    //Get ScannerAddress
    private String getScannerAddress() {
        String address = "";
        EditText editText_ScannerAddress = findViewById(R.id.editText_ScannerAddress);

        if (editText_ScannerAddress != null) {
            address = editText_ScannerAddress.getText().toString();
        }
        return address;
    }

    //Get ScannerPort
    private String getScannerPort() {
        String scannerport = "";
        EditText editText_ScannerPort = findViewById(R.id.editText_ScannerPort);

        if (editText_ScannerPort != null) {
            scannerport = editText_ScannerPort.getText().toString();
        }
        return scannerport;
    }

    //
    // Set Address
    private void setAddress(String address) {
        EditText editText_Address = findViewById(R.id.editText_Address);
        EditText editText_DisplayAddress = findViewById(R.id.editText_DisplayAddress);
        EditText editText_ScannerAddress = findViewById(R.id.editText_ScannerAddress);

        if (editText_Address != null) {
            editText_Address.setText(address);
            editText_DisplayAddress.setText(address);
            editText_ScannerAddress.setText(address);
        }
    }


    //
    // Save & Load Address
    private void saveAddress( int portType, String address )
    {
        switch ( portType )
        {
            case ESCPOSConst.CMP_PORT_WiFi:
                savePreferences( PRE_KEY_IP_ADDRESS, address );
                break;
            case ESCPOSConst.CMP_PORT_Bluetooth:
            case ESCPOSConst.CMP_PORT_Bluetooth_Insecure:
                savePreferences( PRE_KEY_BLUETOOTH_ADDRESS, address );
                break;
            default:
                break;
        }
    }


    private String loadAddress( int portType )
    {
        String address = "";
        switch ( portType )
        {
            case ESCPOSConst.CMP_PORT_WiFi:
                address = loadPreferences( PRE_KEY_IP_ADDRESS, DEFAULT_IP_ADDRESS );
                break;
            case ESCPOSConst.CMP_PORT_Bluetooth:
            case ESCPOSConst.CMP_PORT_Bluetooth_Insecure:
                address = loadPreferences( PRE_KEY_BLUETOOTH_ADDRESS, DEFAULT_BLUETOOTH_ADDRESS );
                break;
            default:
                break;
        }
        return address;
    }

    private void savePreferences( String sKeyName, String sValue )
    {
        SharedPreferences sp = getSharedPreferences( PREFERENCES_NAME, Activity.MODE_PRIVATE );
        SharedPreferences.Editor e = sp.edit();
        e.putString( sKeyName, sValue );
        e.commit();
    }
    private String loadPreferences( String sKeyName, String sDefaultValue )
    {
        SharedPreferences sp = getSharedPreferences( PREFERENCES_NAME, Activity.MODE_PRIVATE );
        return sp.getString( sKeyName, sDefaultValue );
    }

    private void toast( String message, int duration )
    {
        Toast toast = Toast.makeText( getApplicationContext(), message, duration );
        toast.setGravity( Gravity.CENTER, 0, 0 );
        toast.show();
    }

    private void alertDialog( String title, String message )
    {
        new AlertDialog.Builder( this )
                .setTitle( title )
                .setMessage( message )
                .setPositiveButton( "OK", null )
                .show();
    }

    /** requestCode for {@link ActivityCompat.OnRequestPermissionsResultCallback} @Override#onRequestPermissionsResult*/
    private static final int PERMISSION_REQUEST_CODE = 1234;


    /**
     * Get the permissions.
     * @param permissions ex. {android.Manifest.permission.ACCESS_COARSE_LOCATION, android.Manifest.permission.WRITE_EXTERNAL_STORAGE,}
     */
    private void getPermissions(@NonNull final String[] permissions) {

        if (!checkPermissions(permissions)) {
            requestPermissions(permissions);
        } else {
            Log.d("getPermissions","PERMISSION_GRANTED");
        }
    }
    /**
     * Checks whether this app has all permissions.<br>
     * @param permissions ex. {android.Manifest.permission.ACCESS_COARSE_LOCATION, android.Manifest.permission.WRITE_EXTERNAL_STORAGE,}
     * @return true:All permissions granted, false:Permission denied
     */
    private boolean checkPermissions(@NonNull final String[] permissions) {

        for (String permission : permissions) {
            int permissionResult = ContextCompat.checkSelfPermission(this, permission);
////            addResultView(permission + ":" + permissionResult);
            if (permissionResult != android.content.pm.PackageManager.PERMISSION_GRANTED) {
                return false;				// non-permission
            }
        }
        return true;
    }

    /**
     * Requests permissions to be granted to this app.<br>
     * @param permissions ex. {android.Manifest.permission.ACCESS_COARSE_LOCATION, android.Manifest.permission.WRITE_EXTERNAL_STORAGE,}
     */
    private void requestPermissions(@NonNull final String[] permissions) {
        ActivityCompat.requestPermissions(this, permissions, PERMISSION_REQUEST_CODE);
    }
    /**
     * Permissions
     * @see <a href="https://developer.android.com/guide/topics/security/permissions#perm-groups">Dangerous permissions and permission groups.</a>
     */
    private String[] permissions(){
        List<String> list=new ArrayList<>();

        // for Bluetooth
        if (Build.VERSION_CODES.S <= Build.VERSION.SDK_INT) {
            // Android 12 (API-Level 31) -
            list.add(android.Manifest.permission.BLUETOOTH_SCAN);
            list.add(android.Manifest.permission.BLUETOOTH_CONNECT);
        } else
        if (Build.VERSION_CODES.Q <= Build.VERSION.SDK_INT) {
            // Android 10 (API-Level 29) - Android 11 (API-Level 30)
            list.add(android.Manifest.permission.ACCESS_FINE_LOCATION);
        } else
        if (Build.VERSION_CODES.M <= Build.VERSION.SDK_INT) {
            // Android 6.0 (API-Level 23) - Android 9.0 (API-Level 28)
            list.add(android.Manifest.permission.ACCESS_COARSE_LOCATION);
        }

        // for Logging function
        if((Build.VERSION_CODES.M<=Build.VERSION.SDK_INT) &&(Build.VERSION.SDK_INT<=Build.VERSION_CODES.Q)) {
            // Android 6/API level 23 - Android 10/API level 29
            list.add(android.Manifest.permission.WRITE_EXTERNAL_STORAGE);
        }

        return list.toArray(new String[0]);
    }
}
