package com.citizen.app.posapplicationjp;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.hardware.usb.UsbDevice;
import android.os.Build;
import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import android.text.format.DateFormat;
import android.util.Log;
import android.view.View;
import android.view.WindowManager;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.Spinner;
import android.widget.Toast;
import java.util.Calendar;
import java.util.Locale;

import com.citizen.sdk.ESCPOSConst;
import com.citizen.sdk.ESCPOSPrinter;
import com.citizen.sdk.CitizenPrinterInfo;
import	java.util.List;
import	java.util.ArrayList;


public class MainActivity extends Activity {
    private ProgressDialog mProgressDialog;
    private int count = 0;          // 連番の初期化

    // オーダーデータ
    private static class OrderItem {
        public int Id;
        public String Name;
        public int Price;
        public int Num;

        public OrderItem(int id, String name, int price, int num) {
            this.Id = id;
            this.Name = name;
            this.Price = price;
            this.Num = num;
        }
    }

    private final OrderItem[] orders = {
            new OrderItem(1001, "ハンバーガー", 300, 4),
            new OrderItem(1002, "ポテト", 200, 2),
            new OrderItem(1003, "オレンジジュース", 120, 2),
            new OrderItem(1004, "リンゴジュース", 150, 1),
            new OrderItem(1005, "コーラ", 100, 2),
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        this.getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);
        setContentView(R.layout.activity_main);

        mProgressDialog = new ProgressDialog(this);


        getPermissions(permissions());


        // Select IF
        final Spinner spinner = findViewById(R.id.spinner);
        if (spinner != null) {
            spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
                @Override
                public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                    String address = "";
                    int connectType;

                    connectType = getSelectedInterface();
                    if (ESCPOSConst.CMP_PORT_USB == connectType) {
                        address = "null(Automatic Detection)";
                    }
                    setAddress(address);
                }

                @Override
                public void onNothingSelected(AdapterView<?> parent) {
                    Log.e("OnNothingSelected", "NothingSelected");
                }
            });
        }

        // Search
        final Button button_Search = findViewById(R.id.button_Search);
        if (button_Search != null) {
            button_Search.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {

                    int searchMode = 0;			// SearchMode == 0 : searchCitizenPrinter()
                    							// SearchMode != 0 : searchESCPOSPrinter()
                    int connectType;
                    int timeout;
                    int[] errCode = new int[1];

                    // Create an instance( LabelPrinter class )
                    ESCPOSPrinter printer = new ESCPOSPrinter();

                    // Get selected interface
                    connectType = getSelectedInterface();
                    if (connectType == ESCPOSConst.CMP_PORT_USB) {
                        // USB interface is Auto Detect only
                        Toast.makeText(MainActivity.this, "USB interface is Automatic Detection only", Toast.LENGTH_LONG).show();
                        return;
                    }

                    // Set context
                    printer.setContext(MainActivity.this);

                    // Get timeout
                    timeout = getSearchTime(connectType);

                    mProgressDialog.show("Searching...");

                    new Thread(() -> {

                        // Search
                        if (searchMode == 0) {
                            // Search Citizen Printer
                            CitizenPrinterInfo[] list1 = printer.searchCitizenPrinter(connectType, timeout, errCode);
                            //
                            mProgressDialog.dismiss();

                            if (list1.length > 0) {

                                final CharSequence[] item = new CharSequence[list1.length];
                                for (int i = 0; i < list1.length; i++) {
                                    if (connectType == ESCPOSConst.CMP_PORT_WiFi) {
                                        item[i] = list1[i].ipAddress;
                                    } else {
                                        item[i] = list1[i].bdAddress;
                                    }
                                }
                                runOnUiThread(() -> {
                                    // Search address preview and select
                                    new AlertDialog.Builder(MainActivity.this)
                                            .setTitle("Select the address")
                                            .setItems(item,
                                                    new DialogInterface.OnClickListener() {
                                                        @Override
                                                        public void onClick(DialogInterface dialog, int which) {
                                                            setAddress(String.format("%s", item[which]));
                                                        }
                                                    })
                                            .show();
                                });
                            } else {
                                runOnUiThread(() -> {
                                    // 0 search results
                                    Toast.makeText(MainActivity.this, "0 Search Results", Toast.LENGTH_LONG).show();
                                });
                            }
                        } else {
                            // Search Printer
                            String[] list2 = printer.searchESCPOSPrinter(connectType, timeout, errCode);
                            //
                            mProgressDialog.dismiss();

                            if (list2.length > 0) {
                                final CharSequence[] item = list2;
                                runOnUiThread(() -> {
                                    // Search address preview and select
                                    new AlertDialog.Builder(MainActivity.this)
                                            .setTitle("Select the address")
                                            .setItems(item,
                                                    new DialogInterface.OnClickListener() {
                                                        @Override
                                                        public void onClick(DialogInterface dialog, int which) {
                                                            setAddress(String.format("%s", item[which]));
                                                        }
                                                    })
                                            .show();
                                });
                            } else {
                                runOnUiThread(() -> {
                                    // 0 search results
                                    Toast.makeText(MainActivity.this, "0 Search Results", Toast.LENGTH_LONG).show();
                                });
                            }
                        }

                    }).start();
                }
            });
        }

        // レシートボタン
        final Button button_sample1 = findViewById(R.id.button_sample1);
        if (button_sample1 != null) button_sample1.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                //
                mProgressDialog.show("Connecting...");

                new Thread(() -> {

                    int result;
                    int total_Num = 0;
                    int amount;
                    int total_amount = 0;
                    int Tax;
                    int Total;
                    final double TAX_RATE = 0.08;
                    android.graphics.Bitmap bitmap;

                    int wordcount1 = 5;
                    int wordcount2 = 16;
                    int wordcount3 = 3;
                    int wordcount4 = 8;
                    int wordcount5 = 9;
                    int wordcount6 = 7;

                    RadioButton radioButton2 = findViewById(R.id.radioButton2);
                    boolean checked3inch = radioButton2.isChecked();
                    if (checked3inch) {
                        wordcount1 = 6;
                        wordcount2 = 24;
                        wordcount3 = 6;
                        wordcount4 = 12;
                        wordcount5 = 14;
                        wordcount6 = 10;
                    }

                    // インスタンス生成
                    ESCPOSPrinter printer = new ESCPOSPrinter();
                    // 接続
                    result = Connect(printer);

                    if (result != ESCPOSConst.CMP_SUCCESS) {
                        // Error
                        mProgressDialog.dismiss();
                        return;
                    }

                    // 日時を取得
                    String strDATE = (String) DateFormat.format("yyyy-MM-dd  kk:mm:ss", Calendar.getInstance());
                    // 連番を更新
                    count = count + 1;
                    // エンコード設定
                    printer.setEncoding("Shift_JIS");
                    // 一括処理開始設定
                    printer.transactionPrint(ESCPOSConst.CMP_TP_TRANSACTION);
                    // drawble フォルダーから画像ファイルを読み込み
                    bitmap = android.graphics.BitmapFactory.decodeResource(getResources(), R.drawable.shoplogo);

                    // キャッシュ ドロワーをオープン（オプション ブザーを鳴らす）
                    printer.openDrawer(ESCPOSConst.CMP_DRAWER_1, 1);

                    // テキスト印刷
                    printer.printBitmap(bitmap,
                            ESCPOSConst.CMP_BM_ASIS,
                            ESCPOSConst.CMP_ALIGNMENT_CENTER,
                            ESCPOSConst.CMP_BM_MODE_HT_DITHER | ESCPOSConst.CMP_BM_MODE_CMD_RASTER);

                    printer.printText("\n毎度ありがとうございます\n" + "またのご来店お待ちしております\n",
                            ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_BOLD,
                            ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                    printer.printText("ＡＢＣ店  " + strDATE + "\n\n",
                            ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT,
                            ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);


                    for (int i = 0; i < orders.length; i++) {
                        printer.printPaddingText(String.valueOf(orders[i].Id),
                                ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH, wordcount1, ESCPOSConst.CMP_SIDE_RIGHT);

                        printer.printPaddingText(orders[i].Name,
                                ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH, wordcount2, ESCPOSConst.CMP_SIDE_RIGHT);

                        printer.printPaddingText(String.valueOf(orders[i].Num),
                                ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH, wordcount3, ESCPOSConst.CMP_SIDE_LEFT);

                        amount = orders[i].Price * orders[i].Num;

                        printer.printPaddingText("\\" + amount, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH, wordcount4, ESCPOSConst.CMP_SIDE_LEFT);

                        printer.printNormal("\n");

                        total_Num += orders[i].Num;

                        total_amount += amount;
                    }
                    Tax = (int) (total_amount * TAX_RATE);

                    Total = (total_amount + Tax);

                    printer.printNormal("\n");

                    printer.printPaddingText("税　抜",
                            ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_2WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT,
                            wordcount5, ESCPOSConst.CMP_SIDE_RIGHT);

                    printer.printPaddingText("\\" + total_amount,
                            ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1HEIGHT,
                            wordcount6 * 2, ESCPOSConst.CMP_SIDE_LEFT);

                    printer.printNormal("\n");

                    printer.printPaddingText("消費税",
                            ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_2WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT,
                            wordcount5, ESCPOSConst.CMP_SIDE_RIGHT);

                    printer.printPaddingText("\\" + Tax,
                            ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1HEIGHT,
                            wordcount6 * 2, ESCPOSConst.CMP_SIDE_LEFT);

                    printer.printNormal("\n");

                    printer.printPaddingText("合　計",
                            ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_2WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT,
                            wordcount5, ESCPOSConst.CMP_SIDE_RIGHT);

                    printer.printPaddingText("\\" + Total,
                            ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_2WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT,
                            wordcount6, ESCPOSConst.CMP_SIDE_LEFT);

                    printer.printNormal("\n");

                    printer.printText(total_Num + "点　　" + String.format(Locale.US, "%05d", count) + "　  田中\n\n",
                            ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT,
                            ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                    // カット位置紙送り後、パーシャルカット
                    printer.cutPaper(ESCPOSConst.CMP_CUT_PARTIAL_PREFEED);
                    // 一括処理印刷指定
                    result = printer.transactionPrint(ESCPOSConst.CMP_TP_NORMAL);
                    // 切断処理
                    printer.disconnect();
                    //
                    mProgressDialog.dismiss();

                    int finalResult = result;
                    runOnUiThread(() -> {
                        Toast.makeText(MainActivity.this, showMessage(finalResult), Toast.LENGTH_SHORT).show();
                    });

                }).start();
            }
        });

        // 領収書ボタン
        final Button button_sample2 = findViewById(R.id.button_sample2);
        if (button_sample2 != null) {
            button_sample2.setOnClickListener(new View.OnClickListener() {
                public void onClick(View v) {
                    //
                    mProgressDialog.show("Connecting...");

                    new Thread(() -> {

                        int result;
                        int amount;
                        int total_amount = 0;
                        int Tax;
                        int Total;
                        final double TAX_RATE = 0.08;

                        // インスタンス生成
                        ESCPOSPrinter printer = new ESCPOSPrinter();
                        // 接続
                        result = Connect(printer);

                        if (result != ESCPOSConst.CMP_SUCCESS) {
                            // Error
                            mProgressDialog.dismiss();
                            return;
                        }

                        // 日時を取得
                        String strDATE = (String) DateFormat.format("yyyy年MM月dd日", Calendar.getInstance());
                        // 連番を更新
                        count = count + 1;
                        // エンコード設定
                        printer.setEncoding("Shift_JIS");
                        // 一括処理開始設定
                        printer.transactionPrint(ESCPOSConst.CMP_TP_TRANSACTION);

                        // テキスト印刷
                        printer.printText("領 収 書",
                                ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_2WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT);

                        printer.printText("  No." + String.format(Locale.US, "%05d", count) + "\n\n" + strDATE,
                                ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printText("\n\n            　　    様 \n\n",
                                ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_UNDERLINE | ESCPOSConst.CMP_FNT_BOLD,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        for (int i = 0; i < orders.length; i++) {
                            amount = orders[i].Price * orders[i].Num;
                            total_amount += amount;
                        }

                        Tax = (int) (total_amount * TAX_RATE);

                        Total = (total_amount + Tax);

                        printer.printText("  \\    " + Total + "- \n\n",
                                ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_UNDERLINE | ESCPOSConst.CMP_FNT_BOLD,
                                ESCPOSConst.CMP_TXT_2WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT);

                        printer.printText("但し　　　  　  　   \n",
                                ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_UNDERLINE | ESCPOSConst.CMP_FNT_BOLD,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printText("○○○株式会社\n",
                                ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT);

                        printer.printText("○○市○○○　×-××-××\n" + "TEL:XXXX-XX-XXXX\n" + "                       印\n\n",
                                ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        // カット位置紙送り後、パーシャルカット
                        printer.cutPaper(ESCPOSConst.CMP_CUT_PARTIAL_PREFEED);
                        // 一括処理印字指定
                        result = printer.transactionPrint(ESCPOSConst.CMP_TP_NORMAL);
                        // 切断処理
                        printer.disconnect();
                        //
                        mProgressDialog.dismiss();

                        int finalResult = result;
                        runOnUiThread(() -> {
                            Toast.makeText(MainActivity.this, showMessage(finalResult), Toast.LENGTH_SHORT).show();
                        });

                    }).start();
                }
            });
        }

        // 調理指示単品伝票ボタン
        final Button button_sample3 = findViewById(R.id.button_sample3);
        button_sample3.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                //
                mProgressDialog.show("Connecting...");

                new Thread(() -> {

                    int result;

                    int wordcount1 = 18;
                    int wordcount2 = 9;

                    RadioButton radioButton2 = findViewById(R.id.radioButton2);
                    boolean checked3inch = radioButton2.isChecked();
                    if (checked3inch) {
                        wordcount1 = 26;
                        wordcount2 = 13;
                    }

                    // インスタンス生成
                    ESCPOSPrinter printer = new ESCPOSPrinter();
                    // 接続
                    result = Connect(printer);

                    if (result != ESCPOSConst.CMP_SUCCESS) {
                        // Error
                        mProgressDialog.dismiss();
                        return;
                    }

                    // 日時を取得
                    String strTime = (String) DateFormat.format("kk:mm", Calendar.getInstance());
                    // 連番を更新
                    count = count + 1;
                    // エンコード設定
                    printer.setEncoding("Shift_JIS");
                    // 一括処理開始設定
                    printer.transactionPrint(ESCPOSConst.CMP_TP_TRANSACTION);
                    // ブザーを鳴らす （ブザー対応プリンターが必要です）
                    byte[] data = {0x1b, 0x1e, 0x00, 0x1b, 0x1e, 0x00, 0x1b, 0x1e, 0x00};
                    printer.printData(data);

                    // テキスト印刷
                    for (int i = 0; i < orders.length; i++) {
                        printer.printText(orders[i].Num + "  ",
                                ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_2WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT);

                        printer.printPaddingText(orders[i].Name,
                                ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT,
                                wordcount1, ESCPOSConst.CMP_SIDE_RIGHT);

                        printer.printNormal("\n");

                        printer.printText("調理単品",
                                ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT | ESCPOSConst.CMP_FNT_REVERSE,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printPaddingText("A03", ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_2WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT,
                                wordcount2, ESCPOSConst.CMP_SIDE_LEFT);

                        printer.printNormal("\n");

                        printer.printText(String.format(Locale.US, "%05d", count) + "    2名    田中    " + strTime + "\n",
                                ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        // 連番を更新
                        count = count + 1;
                        // カット位置紙送り後、パーシャルカット
                        printer.cutPaper(ESCPOSConst.CMP_CUT_PARTIAL_PREFEED);
                    }

                    // 一括処理印刷指定
                    result = printer.transactionPrint(ESCPOSConst.CMP_TP_NORMAL);
                    // 切断処理
                    printer.disconnect();
                    //
                    mProgressDialog.dismiss();

                    int finalResult = result;
                    runOnUiThread(() -> {
                        Toast.makeText(MainActivity.this, showMessage(finalResult), Toast.LENGTH_SHORT).show();
                    });

                }).start();
            }
        });


        // 調理指示一枚伝票ボタン
        final Button button_sample4 = findViewById(R.id.button_sample4);
        if (button_sample4 != null) {
            button_sample4.setOnClickListener(new View.OnClickListener() {
                public void onClick(View v) {
                    //
                    mProgressDialog.show("Connecting...");

                    new Thread(() -> {

                        int result;

                        int wordcount1 = 30;
                        int fontsize1 = ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT;

                        RadioButton radioButton2 = findViewById(R.id.radioButton2);
                        boolean checked3inch = radioButton2.isChecked();
                        if (checked3inch) {
                            wordcount1 = 46;
                            fontsize1 = ESCPOSConst.CMP_TXT_2WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT;
                        }

                        // インスタンス生成
                        ESCPOSPrinter printer = new ESCPOSPrinter();
                        // 接続
                        result = Connect(printer);

                        if (result != ESCPOSConst.CMP_SUCCESS) {
                            // Error
                            mProgressDialog.dismiss();
                            return;
                        }

                        // 日時を取得
                        String strTIME = (String) DateFormat.format(" kk:mm", Calendar.getInstance());
                        // 連番を更新
                        count = count + 1;
                        // エンコード設定
                        printer.setEncoding("Shift_JIS");
                        // 一括処理開始設定
                        printer.transactionPrint(ESCPOSConst.CMP_TP_TRANSACTION);
                        // ブザーを鳴らす （ブザー対応プリンターが必要です）
                        byte[] data = {0x1b, 0x1e, 0x00, 0x1b, 0x1e, 0x00, 0x1b, 0x1e, 0x00};
                        printer.printData(data);

                        // テキスト印刷
                        printer.printText("調理一枚\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_REVERSE,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printText("テーブル",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printPaddingText(" A03 ",
                                ESCPOSConst.CMP_FNT_DEFAULT, fontsize1,
                                7, ESCPOSConst.CMP_SIDE_LEFT);

                        printer.printText("   伝　票 ",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printPaddingText(String.format(Locale.US, "%05d", count) + "\n",
                                ESCPOSConst.CMP_FNT_DEFAULT, fontsize1,
                                8, ESCPOSConst.CMP_SIDE_LEFT);

                        printer.printText("人　数  ",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printPaddingText(" 2 ",
                                ESCPOSConst.CMP_FNT_DEFAULT, fontsize1,
                                7, ESCPOSConst.CMP_SIDE_LEFT);

                        printer.printText("   担　当 ",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printPaddingText("田中\n",
                                ESCPOSConst.CMP_FNT_DEFAULT, fontsize1,
                                8, ESCPOSConst.CMP_SIDE_LEFT);

                        printer.printText("時　刻  ",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printPaddingText(" " + strTIME + " ",
                                ESCPOSConst.CMP_FNT_DEFAULT, fontsize1,
                                7, ESCPOSConst.CMP_SIDE_LEFT);

                        printer.printText("   HT番号 ",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printPaddingText("HT001\n",
                                ESCPOSConst.CMP_FNT_DEFAULT, fontsize1,
                                8, ESCPOSConst.CMP_SIDE_LEFT);

                        printer.setRecLineSpacing(24);

                        printer.printPaddingText("┏━┯━━┯━━━━━━━━━━━━━━━━━",
                                ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH, wordcount1, ESCPOSConst.CMP_SIDE_RIGHT);

                        printer.printText("┓\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printPaddingText("┃　│数量│　　品　　名",
                                ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH, wordcount1, ESCPOSConst.CMP_SIDE_RIGHT);

                        printer.printText("┃\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printPaddingText("┠─┼──┼─────────────────",
                                ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH, wordcount1, ESCPOSConst.CMP_SIDE_RIGHT);

                        printer.printText("┨\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        for (int i = 0; i < orders.length; i++) {
                            printer.printText("┃",
                                    ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                    ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT);

                            printer.printText("□",
                                    ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                    ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                            printer.printText("│",
                                    ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                    ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT);

                            printer.printPaddingText(String.valueOf(orders[i].Num),
                                    ESCPOSConst.CMP_FNT_DEFAULT,
                                    ESCPOSConst.CMP_TXT_2WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT,
                                    2, ESCPOSConst.CMP_SIDE_LEFT);

                            printer.printPaddingText("│" + orders[i].Name,
                                    ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT,
                                    wordcount1 - 10, ESCPOSConst.CMP_SIDE_RIGHT);

                            printer.printText("┃\n",
                                    ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                    ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT);
                        }

                        printer.printPaddingText("┃　│    │                            ",
                                ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH, wordcount1, ESCPOSConst.CMP_SIDE_RIGHT);

                        printer.printText("┃\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printPaddingText("┃　│    │                            ",
                                ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH, wordcount1, ESCPOSConst.CMP_SIDE_RIGHT);

                        printer.printText("┃\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printPaddingText("┃　│    │                            ",
                                ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH, wordcount1, ESCPOSConst.CMP_SIDE_RIGHT);

                        printer.printText("┃\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printPaddingText("┗━┷━━┷━━━━━━━━━━━━━━━━━",
                                ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH, wordcount1, ESCPOSConst.CMP_SIDE_RIGHT);

                        printer.printText("┛",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.setRecLineSpacing(34);

                        // カット位置紙送り後、パーシャルカット
                        printer.cutPaper(ESCPOSConst.CMP_CUT_PARTIAL_PREFEED);
                        // 一括処理印刷指定
                        result = printer.transactionPrint(ESCPOSConst.CMP_TP_NORMAL);
                        // 切断処理
                        printer.disconnect();
                        //
                        mProgressDialog.dismiss();

                        int finalResult = result;
                        runOnUiThread(() -> {
                            Toast.makeText(MainActivity.this, showMessage(finalResult), Toast.LENGTH_SHORT).show();
                        });

                    }).start();
                }
            });
        }

        // ゲスト伝票ボタン
        final Button button_sample5 = findViewById(R.id.button_sample5);
        button_sample5.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                //
                mProgressDialog.show("Connecting...");

                new Thread(() -> {

                    int result;
                    int amount;
                    int total_amount = 0;
                    int Tax = 0;
                    int Total = 0;
                    final double TAX_RATE = 0.08;
                    android.graphics.Bitmap bitmap;

                    RadioButton radioButton2 = findViewById(R.id.radioButton2);
                    boolean checked3inch = radioButton2.isChecked();

                    // インスタンス生成
                    ESCPOSPrinter printer = new ESCPOSPrinter();
                    // 接続
                    result = Connect(printer);

                    if (result != ESCPOSConst.CMP_SUCCESS) {
                        // Error
                        mProgressDialog.dismiss();
                        return;
                    }

                    // 日時を取得
                    String strDATE = (String) DateFormat.format("yyyy-MM-dd", Calendar.getInstance());
                    String strTIME = (String) DateFormat.format("kk:mm", Calendar.getInstance());
                    // 連番を更新
                    count = count + 1;
                    // エンコード設定
                    printer.setEncoding("Shift_JIS");
                    // 一括処理開始設定
                    printer.transactionPrint(ESCPOSConst.CMP_TP_TRANSACTION);
                    // ページモードの開始
                    printer.pageModePrint(ESCPOSConst.CMP_PM_PAGE_MODE);
                    //// ページモードのオフセットの指定
                    printer.setPageModeVerticalPosition(0);
                    printer.setPageModeHorizontalPosition(0);
                    // ページモードの印刷方向の指定
                    printer.setPageModePrintDirection(ESCPOSConst.CMP_PD_LEFT_TO_RIGHT);
                    // drawble フォルダーから画像ファイルを読み込み
                    bitmap = android.graphics.BitmapFactory.decodeResource(getResources(), R.drawable.shoplogo);

                    // テキスト印刷
                    if (checked3inch) {
                        printer.setPageModePrintArea("0,0,300,152");

                        printer.printText("【新　規】\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.setPageModeVerticalPosition(32);
                        printer.printText(strDATE,
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.setPageModeVerticalPosition(64);
                        printer.printBitmap(bitmap,
                                280, ESCPOSConst.CMP_ALIGNMENT_LEFT,
                                ESCPOSConst.CMP_BM_MODE_HT_DITHER | ESCPOSConst.CMP_BM_MODE_CMD_RASTER);

                        printer.setPageModePrintArea("320,0,210,152");

                        printer.setPageModeHorizontalPosition(0);
                        printer.setPageModeVerticalPosition(0);

                        printer.printText("伝　票　　  " + String.format(Locale.US, "%05d", count) + "\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.setPageModeVerticalPosition(32);
                        printer.printText("テーブル　　A03\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.setPageModeVerticalPosition(64);
                        printer.printText("人　数　　  2\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.setPageModeVerticalPosition(96);
                        printer.printText("時　刻　　  " + strTIME,
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.setPageModeVerticalPosition(128);
                        printer.printText("担　当　　  田中\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.pageModePrint(ESCPOSConst.CMP_PM_NORMAL);

                        printer.setRecLineSpacing(24);

                        printer.printText("┏━━━━━━━━━━━━━┯━━┯━━━━━┓\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printText("┃　　　品　　　　　名　　　│数量│　金　額　┃\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printText("┠─────────────┼──┼─────┨\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        for (int i = 0; i < orders.length; i++) {
                            printer.printText("┃",
                                    ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT);

                            printer.printPaddingText(String.valueOf(orders[i].Id),
                                    ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT, 6, ESCPOSConst.CMP_SIDE_RIGHT);

                            printer.printPaddingText(orders[i].Name,
                                    ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT, 19, ESCPOSConst.CMP_SIDE_RIGHT);

                            printer.printPaddingText("│   " + orders[i].Num,
                                    ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT, 7, ESCPOSConst.CMP_SIDE_LEFT);

                            printer.printPaddingText("│       " + orders[i].Price,
                                    ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT, 12, ESCPOSConst.CMP_SIDE_LEFT);

                            printer.printText("┃\n",
                                    ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT);

                            amount = orders[i].Price * orders[i].Num;

                            total_amount += amount;
                        }

                        Tax = (int) (total_amount * TAX_RATE);

                        Total = (total_amount + Tax);

                        printer.printText("┃　　　　　　　　　　　　　│　　│　　　　　┃\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printText("┣━━━━━┯━━━━┯━━┷━━┿━━━━━┫\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printText("┃　小　計　│",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printPaddingText(String.valueOf(total_amount),
                                ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1HEIGHT, 8, ESCPOSConst.CMP_SIDE_LEFT);

                        printer.printText("│消　費　税│",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printPaddingText(String.valueOf(Tax), ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1HEIGHT, 10, ESCPOSConst.CMP_SIDE_LEFT);

                        printer.printText("┃\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printText("┠─────┴────┼─────┴─────┨\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printText("┃　合　計　          │",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT);

                        printer.printPaddingText(String.valueOf(Total), ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT, 22, ESCPOSConst.CMP_SIDE_LEFT);

                        printer.printText("┃\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT);

                        printer.printText("┗━━━━━━━━━━┷━━━━━━━━━━━┛\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);
                    } else {

                        printer.setPageModePrintArea("0,0,200,152");

                        printer.printText("【新　規】\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.setPageModeVerticalPosition(32);
                        printer.printText(strDATE,
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.setPageModeVerticalPosition(72);
                        printer.printBitmap(bitmap,
                                200, ESCPOSConst.CMP_ALIGNMENT_LEFT,
                                ESCPOSConst.CMP_BM_MODE_HT_DITHER | ESCPOSConst.CMP_BM_MODE_CMD_RASTER);

                        printer.setPageModePrintArea("210,0,170,152");

                        printer.setPageModeHorizontalPosition(0);
                        printer.setPageModeVerticalPosition(0);

                        printer.printText("伝　票　 " + String.format(Locale.US, "%05d", count) + "\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.setPageModeVerticalPosition(32);
                        printer.printText("ﾃｰﾌﾞﾙ　  A03\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.setPageModeVerticalPosition(64);
                        printer.printText("人　数　 2\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.setPageModeVerticalPosition(96);
                        printer.printText("時　刻　 " + strTIME,
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.setPageModeVerticalPosition(128);
                        printer.printText("担　当　 田中\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.pageModePrint(ESCPOSConst.CMP_PM_NORMAL);

                        printer.setRecLineSpacing(24);

                        printer.printText("┏━━━━━━━━━┯━┯━━┓\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printText("┃　　品　　名　　　│数│金額┃\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printText("┠─────────┼─┼──┨\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        for (int i = 0; i < orders.length; i++) {
                            printer.printText("┃",
                                    ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT);

                            printer.printPaddingText(String.valueOf(orders[i].Id),
                                    ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT, 5, ESCPOSConst.CMP_SIDE_RIGHT);

                            printer.printPaddingText(orders[i].Name,
                                    ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT, 12, ESCPOSConst.CMP_SIDE_RIGHT);

                            printer.printPaddingText("│" + orders[i].Num,
                                    ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT, 4, ESCPOSConst.CMP_SIDE_LEFT);

                            printer.printPaddingText("│ " + orders[i].Price,
                                    ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT, 7, ESCPOSConst.CMP_SIDE_LEFT);

                            printer.printText("┃\n",
                                    ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT);

                            amount = orders[i].Price * orders[i].Num;

                            total_amount += amount;
                        }

                        Tax = (int) (total_amount * TAX_RATE);

                        Total = (total_amount + Tax);

                        printer.printText("┃　　　　　　　　　│　│　　┃\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printText("┣━━━┯━━━┯━┷━┿━━┫\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printText("┃小　計│",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printPaddingText(String.valueOf(total_amount),
                                ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1HEIGHT, 6, ESCPOSConst.CMP_SIDE_LEFT);

                        printer.printText("│消費税│",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printPaddingText(String.valueOf(Tax),
                                ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1HEIGHT, 4, ESCPOSConst.CMP_SIDE_LEFT);

                        printer.printText("┃\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printText("┠───┴───┼───┴──┨\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                        printer.printText("┃　合　計　　　│",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT);

                        printer.printPaddingText(String.valueOf(Total),
                                ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT, 12, ESCPOSConst.CMP_SIDE_LEFT);

                        printer.printText("┃\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT);

                        printer.printText("┗━━━━━━━┷━━━━━━┛\n",
                                ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_FNT_DEFAULT,
                                ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);
                    }

                    printer.setRecLineSpacing(34);

                    printer.printText("毎度ありがとうございます\n" + "またのご来店をお待ちしております\n",
                            ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT,
                            ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                    printer.printBarCode("123456789012",
                            ESCPOSConst.CMP_BCS_UPCA, 50, 2,
                            ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_HRI_TEXT_NONE);
                    printer.cutPaper(ESCPOSConst.CMP_CUT_PARTIAL_PREFEED);

                    result = printer.transactionPrint(ESCPOSConst.CMP_TP_NORMAL);

                    printer.disconnect();
                    //
                    mProgressDialog.dismiss();

                    int finalResult = result;
                    runOnUiThread(() -> {
                        Toast.makeText(MainActivity.this, showMessage(finalResult), Toast.LENGTH_SHORT).show();
                    });

                }).start();
            }
        });
    }

    // 接続設定
    private int Connect(ESCPOSPrinter printer) {
        int connectType;
        String address;
        int result;

        connectType = getSelectedInterface();
        address = getAddress();

        if (connectType == ESCPOSConst.CMP_PORT_USB) {
            printer.setContext(MainActivity.this);
            UsbDevice usbDevice = null;
            result = printer.connect(ESCPOSConst.CMP_PORT_USB, usbDevice);
        } else {
            result = printer.connect(connectType, address);
        }

        if (result != ESCPOSConst.CMP_SUCCESS) {
            runOnUiThread(() -> {
                // Error
                Toast.makeText(MainActivity.this, showMessage(result), Toast.LENGTH_SHORT).show();
            });
        }
        return result;
    }


    private int getSearchTime(int connectType) {
        int timeout;

        if (ESCPOSConst.CMP_PORT_WiFi == connectType) {
            timeout = 3;
        } else if ((ESCPOSConst.CMP_PORT_Bluetooth == connectType) || (ESCPOSConst.CMP_PORT_Bluetooth_Insecure == connectType)) {
            timeout = 10;
        } else {
            timeout = 0;
        }
        return timeout;
    }

    private int getSelectedInterface() {
        int selectIF = ESCPOSConst.CMP_PORT_WiFi;
        Spinner spinner = findViewById(R.id.spinner);

        if (spinner != null) {
            selectIF = spinner.getSelectedItemPosition();
        }
        return selectIF;
    }

    private String getAddress() {
        String address = "";
        EditText editText_Address = findViewById(R.id.editText_Address);

        if (editText_Address != null) {
            address = editText_Address.getText().toString();
        }
        return address;
    }

    private void setAddress(String address) {
        EditText editText_Address = findViewById(R.id.editText_Address);

        if (editText_Address != null) {
            editText_Address.setText(address);
        }
    }

    private String showMessage(int result) {
        String word;
        switch (result) {
            case ESCPOSConst.CMP_SUCCESS:
                word = "正常終了";
                break;

            case ESCPOSConst.CMP_E_CONNECTED:
                word = "プリンターへ既に接続済みです。";
                break;

            case ESCPOSConst.CMP_E_DISCONNECT:
                word = "プリンターへ接続していません。";
                break;

            case ESCPOSConst.CMP_E_NOTCONNECT:
                word = "プリンターへ接続できませんでした。";
                break;

            case ESCPOSConst.CMP_E_CONNECT_NOTFOUND:
                word = "プリンター接続後の対応機種確認に失敗しました。";
                break;

            case ESCPOSConst.CMP_E_CONNECT_OFFLINE:
                word = "プリンター接続後のプリンター状態確認に失敗しました。";
                break;

            case ESCPOSConst.CMP_E_ILLEGAL:
                word = "サポートされていない処理または無効なパラメータ値です。";
                break;

            case ESCPOSConst.CMP_E_OFFLINE:
                word = "プリンターがオフラインです。";
                break;

            case ESCPOSConst.CMP_E_NOEXIST:
                word = "指定のファイルが存在しません。";
                break;

            case ESCPOSConst.CMP_E_FAILURE:
                word = "要求された処理が実行できません。";
                break;

            case ESCPOSConst.CMP_E_TIMEOUT:
                word = "所定の時間が経過してもプリンターからの応答がありません。";
                break;

            case ESCPOSConst.CMP_E_NO_LIST:
                word = "プリンター検索にてプリンターが見つかりません。";
                break;

            case ESCPOSConst.CMP_EPTR_COVER_OPEN:
                word = "プリンターのカバーが開いています。";
                break;

            case ESCPOSConst.CMP_EPTR_REC_EMPTY:
                word = "用紙切れです。";
                break;

            case ESCPOSConst.CMP_EPTR_BADFORMAT:
                word = "指定されたファイルの書式がサポートされていません。";
                break;

            case ESCPOSConst.CMP_EPTR_TOOBIG:
                word = "指定されたビットマップのサイズが多きすぎます。";
                break;

            default:
                word = "その他のエラーです。";
                break;
        }
        return word;
    }

    /** requestCode for {@link ActivityCompat.OnRequestPermissionsResultCallback} @Override#onRequestPermissionsResult*/
    private static final int PERMISSION_REQUEST_CODE = 1234;


    /**
     * Get the permissions.
     * @param permissions ex. {android.Manifest.permission.ACCESS_COARSE_LOCATION, android.Manifest.permission.WRITE_EXTERNAL_STORAGE,}
     */
    private void getPermissions(@NonNull final String[] permissions) {

        if (!checkPermissions(permissions)) {
            requestPermissions(permissions);
        } else {
            Log.d("getPermissions","PERMISSION_GRANTED");
        }
    }
    /**
     * Checks whether this app has all permissions.<br>
     * @param permissions ex. {android.Manifest.permission.ACCESS_COARSE_LOCATION, android.Manifest.permission.WRITE_EXTERNAL_STORAGE,}
     * @return true:All permissions granted, false:Permission denied
     */
    private boolean checkPermissions(@NonNull final String[] permissions) {

        for (String permission : permissions) {
            int permissionResult = ContextCompat.checkSelfPermission(this, permission);
////            addResultView(permission + ":" + permissionResult);
            if (permissionResult != android.content.pm.PackageManager.PERMISSION_GRANTED) {
                return false;				// non-permission
            }
        }
        return true;
    }
    /**
     * Requests permissions to be granted to this app.<br>
     * @param permissions ex. {android.Manifest.permission.ACCESS_COARSE_LOCATION, android.Manifest.permission.WRITE_EXTERNAL_STORAGE,}
     */
    private void requestPermissions(@NonNull final String[] permissions) {
        ActivityCompat.requestPermissions(this, permissions, PERMISSION_REQUEST_CODE);
    }

    /**
     * Permissions
     * @see <a href="https://developer.android.com/guide/topics/security/permissions#perm-groups">Dangerous permissions and permission groups.</a>
     */
    private String[] permissions(){
        List<String> list=new ArrayList<>();

        // for Bluetooth
        if (Build.VERSION_CODES.S <= Build.VERSION.SDK_INT) {
            // Android 12 (API-Level 31) -
            list.add(android.Manifest.permission.BLUETOOTH_SCAN);
            list.add(android.Manifest.permission.BLUETOOTH_CONNECT);
        } else
        if (Build.VERSION_CODES.Q <= Build.VERSION.SDK_INT) {
            // Android 10 (API-Level 29) - Android 11 (API-Level 30)
            list.add(android.Manifest.permission.ACCESS_FINE_LOCATION);
        } else
        if (Build.VERSION_CODES.M <= Build.VERSION.SDK_INT) {
            // Android 6.0 (API-Level 23) - Android 9.0 (API-Level 28)
            list.add(android.Manifest.permission.ACCESS_COARSE_LOCATION);
        }

        // for Logging function
        if((Build.VERSION_CODES.M<=Build.VERSION.SDK_INT) &&(Build.VERSION.SDK_INT<=Build.VERSION_CODES.Q)) {
            // Android 6/API level 23 - Android 10/API level 29
            list.add(android.Manifest.permission.WRITE_EXTERNAL_STORAGE);
        }

        return list.toArray(new String[0]);
    }
}
